/** 
* @file LocalFeatureRepresentation.h
* @brief Absract class for Local Feature Representations (Detector + Descriptor)
* @author Erik Rodner, Alexander Freytag
* @date 11/19/2007

*/
#ifndef LOCALFEATUREREPRESENTATIONINCLUDE
#define LOCALFEATUREREPRESENTATIONINCLUDE

#include "core/imagedisplay/ImageDisplay.h"

#include "core/vector/VectorT.h"
#include "core/vector/MatrixT.h"
#include "core/image/ImageT.h"
 
#include <vector>
#include "core/vector/VVector.h"


namespace OBJREC {

/** absract class for the representation of an image with local feature descriptors */
/** NOTE: This class returns Descriptors, which DO NOT need any given positions. All Descriptors calculate there own positions, on DIFFERENT ways. **/

  /** @class LocalFeatureRepresentation
 * @brief Absract class for Local Feature Representations (Detector + Descriptor)
 *
 */
class LocalFeatureRepresentation
{

    protected:

    public:
  
	/** simple constructor */
	LocalFeatureRepresentation();
      
	/** simple destructor */
	virtual ~LocalFeatureRepresentation();

	virtual int getDescSize () const = 0;

	virtual int extractFeatures ( const NICE::Image & img, 
				      NICE::VVector & features, 
				      NICE::VVector & positions ) const = 0;
	
	virtual int extractFeatures ( const NICE::ColorImage & img, 
				      NICE::VVector & features, 
				      NICE::VVector & positions ) const;

	virtual void visualizeFeatures ( NICE::Image & mark,
					 const NICE::VVector & positions,
					 size_t color ) const;

	virtual void visualize ( NICE::Image & img, 
				 const NICE::Vector & feature ) const;

};


} // namespace

#endif
